# -*- coding: utf-8 -*-
################################################################################
#
# ROSA Crypto
#
# Copyright (c) 2016, LLC "NTC IT ROSA"
# License: BSD
# Authors:
#     Michl Voznesensky <m.voznesensky@rosalinux.ru>
#
# This file is a part of ROSA Crypto
#
# All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are
#  met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following disclaimer
#    in the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of the LLC "NTC IT ROSA" nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

import sys
import os.path
import subprocess
import rename
import time
#rename nedeed for vefify sign
#files with the name Cyrillic characters are not available to take certificate
import re
import datetime
from PyQt4.QtCore import QUrl, Qt
from PyQt4.QtGui import QDesktopServices, QMessageBox
from pyudev import Context, Monitor
from pyudev.pyqt4 import MonitorObserver

UTILS_CRYPT = ('csptest', 'certmgr', 'cryptcp')

class RosaApp(object):
	""" Logic of the program """
	
	def __init__(self, app_ui):
		self._ui = app_ui #main window
		
		# Monitoring usb Token
		# monitor started after first run checkProgramm
		self.context = Context() # all devices in system
		self.monitor = Monitor.from_netlink(self.context) # synchronous monitoring
		self.monitor.filter_by(subsystem = 'usb') # monitoring usb devices
		observer = MonitorObserver(self.monitor, self._ui) # asynchronous monitoring for devices
		observer.deviceEvent.connect(self.__detectToken)
		
		self.checkProgramm(True)
		
		self._ui.exit_act.triggered.connect(exit)
		
		self._ui.help_act.triggered.connect(self.openHelp)
		
		self._ui.check_comp_act.triggered.connect(self.checkProgramm)
		
		self._ui.sign.btn_sign.clicked.connect(self.createSign)
		self._ui.check_sign.btn_check_sign.clicked.connect(self.verifySignature)
		self._ui.encrypt.btn_encrypt.clicked.connect(self.encryptFile)
		self._ui.decrypt.btn_decrypt.clicked.connect(self.decryptFile)
	
	def checkProgramm(self, first_run = False):
		""" This checks all the necessary components for the successful operation of the program.
		If something is missing you will see the corresponding message """
		
		insert_token = False #check insert token on computer
		
		if not first_run:
			map(lambda field: field.setEnabled(True), self._ui.list_off_all)
			self._ui.sign.list_avail_cert.clear()
			self._ui.encrypt.list_avail_cert.clear()
			self._ui.decrypt.list_avail_cert.clear()
		
		list_comp = []
		
		#check installed CryptoPro fasa 1
		if os.path.isdir('/opt/cprocsp/bin/amd64/'):
			self.dir_crypt = '/opt/cprocsp/bin/amd64/' #directory CryptoPro
		elif os.path.isdir('/opt/cprocsp/bin/ia32/'):
			self.dir_crypt = '/opt/cprocsp/bin/ia32/'
		else: self.dir_crypt = None
		
		if self.dir_crypt is None:
			map(lambda field: field.setEnabled(False), self._ui.list_off_all) #disable almost all components
			list_comp.append("crypto")
		else:
			#check installed CryptoPro fasa 2
			list_utils_crypt = os.listdir(self.dir_crypt)
			check_utils_crypt = map(lambda util: util in list_utils_crypt, UTILS_CRYPT)
			if False in check_utils_crypt:
				map(lambda field: field.setEnabled(False), self._ui.list_off_all)
				list_comp.append("crypto")
			else:
				#check license CryptoPro
				lic_crypt = subprocess.call("%scpconfig -license -view" %self.dir_crypt, shell=True)
				if lic_crypt == 3:
					list_comp.append("license")
					map(lambda field: field.setEnabled(False), self._ui.list_off_lic)
				#chek token
				if self.__listToken():
					insert_token = True
				if not insert_token:
					map(lambda field: field.setEnabled(False), self._ui.list_off_token) #disable components linked to signature
					list_comp.append("token")
				#check running PCSCD (needed for token)
				subprocess.call("%scsptest -keyset -enum_cont -verifyc -fq" %self.dir_crypt, shell=True) #check, automated run pcscd
				run_pcscd = subprocess.call("pidof pcscd", shell=True)
				if run_pcscd != 0 and insert_token:
					list_comp.append("pcsc")
					map(lambda field: field.setEnabled(False), self._ui.list_off_token)
				else:
					if insert_token:
						#Can use subprocess.check_output, but this function work on Python since 2.7
						#To work on python 2.6 uses the old style
						info_cert = subprocess.Popen("%scsptest -keyset -enum_cont -verifyc -fq" %self.dir_crypt, stdout=subprocess.PIPE, shell=True).communicate()[0]
						self.name_cert = self.__parseConsole(info_cert, "\\\\")
						if not self.name_cert:
							map(lambda field: field.setEnabled(False), self._ui.list_off_token) #disable components linked to signature
							list_comp.append("empty_token")
						else:
							self._ui.stat_prog_icon.setPixmap(self._ui.prog_icon_green)
							self._ui.stat_prog_text.setText(_("Token(s) %s is connected") %(', '.join(self.__listToken())))
							self._ui.sign.list_avail_cert.addItems(self.name_cert)
							self._ui.decrypt.list_avail_cert.addItems(self.name_cert)
					if first_run:
						self.monitor.start() #start new flow for checker USB device
		
		#if all_cert empty then triggered exception
		get_cert = subprocess.Popen("%scertmgr -list" %self.dir_crypt, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
		all_cert, err_cert = get_cert.communicate()
		if err_cert != "Empty certificate list":
			self._ui.encrypt.list_avail_cert.addItems(self.__parseConsole(all_cert, "Subject"))
		else:
			self._ui.encrypt.list_avail_cert.addItem(_("(Empty)"))
		
		self.informBox(list_comp, first_mess = first_run)
	
	def openHelp(self):
		QDesktopServices.openUrl(QUrl("/usr/share/doc/rosa-crypto-tool/help.pdf")) #open help in default programm
	
	def infoCert(self, where_cert, mode):
		""" Generates information about the certificate.
		When creating a signature file, certificate is taken from Rutoken.
		When verifying a signature - from a file """
		
		if where_cert == "token":
			status_cert = subprocess.Popen("%scertmgr -list -cont '%s'" %(self.dir_crypt, mode.list_avail_cert.currentText()), stdout=subprocess.PIPE, shell=True).communicate()[0]
		elif where_cert == "file":
			status_cert = subprocess.Popen("%scertmgr -list -file %s" %(self.dir_crypt, mode.file_edit.text()), stdout=subprocess.PIPE, shell=True).communicate()[0]
		elif where_cert == "store":
			status_cert = subprocess.Popen("%scertmgr -list -dn %s" %(self.dir_crypt, mode.list_avail_cert.currentText()), stdout=subprocess.PIPE, shell=True).communicate()[0]
		
		self.name_subj = self.__parseConsole(status_cert, "Subject")[0] #need to sign the files (finding certificate)
		row_valid_cert = self.__parseConsole(status_cert, "Not valid after")[0] #need to check valid certificate
		self.gost = self.__parseConsole(status_cert, "Signature Algorithm")[0]
		
		valid_cert = re.findall('[0-9]{2,4}',row_valid_cert) #take data
		valid_cert = map(lambda x: int(x), valid_cert) #chage type data
		self.date_valid_cert = datetime.datetime(valid_cert[2], valid_cert[1], valid_cert[0], valid_cert[3], valid_cert[4], valid_cert[5])
		
		self.out_status_cert = _("Information about the certificate:\n") +\
							_("- Issuer: ") + self.__parseConsole(status_cert, "Issuer")[0] + "\n" +\
							_("- Subject: ") + self.name_subj + "\n" +\
							_("- Serial: ") + self.__parseConsole(status_cert, "Serial")[0] + "\n" +\
							_("- Signature Algorithm: ") + self.gost + "\n" +\
							_("- PublicKey Algorithm: ") + self.__parseConsole(status_cert, "PublicKey Algorithm")[0] + "\n" +\
							_("- Valid from ") + self.__parseConsole(status_cert, "Not valid before")[0] + _(" to ") + row_valid_cert
	
	def createSign(self):
		""" Create sign for file with the extension .sig """
		
		try:
			if self._ui.sign.file_edit.text():
				self.infoCert("token", self._ui.sign)
				
				new_name = rename.translit(self._ui.sign.file_name) + ".sig"
				
				subprocess.check_call('yes | %scryptcp -dir "%s" -sign -dn "%s" -der "%s" "%s"' %(self.dir_crypt, self._ui.sign.currently_dir, self.name_subj, self._ui.sign.path_name, self._ui.sign.currently_dir + new_name), shell=True)
				
				out_status_sig = _('The file "%s" has been signed.\nCreated a new signature file "%s"') %(self._ui.sign.file_name, new_name)
				
				self.__outMessage(out_status_sig, self._ui.sign.icon_success, self._ui.sign, True)
				
				if self._ui.sign.break_recursion: self._ui.sign.break_recursion = False
			else:
				out_status_sig = _("Please select the file to sign")
				
				self.__outMessage(out_status_sig, self._ui.sign.icon_atten, self._ui.sign, False)
		except subprocess.CalledProcessError:
			if not self._ui.sign.break_recursion:
				self._ui.sign.break_recursion = True
				#Dont't change the single quotation marks in the name of the container, else slashes must be doubled
				get_result_inst_cert = subprocess.Popen("%scertmgr -inst -cont '%s' -store uMy" %(self.dir_crypt, self._ui.sign.list_avail_cert.currentText()), stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
				result_inst_cert, err_result_inst_cert = get_result_inst_cert.communicate()
				if err_result_inst_cert:
					out_status_sig = _('\nError: ') + err_result_inst_cert
				
					self.__outMessage(out_status_sig, self._ui.sign.icon_atten, self._ui.sign, False)
				else:
					self._ui.encrypt.list_avail_cert.insertItems(0, self.__parseConsole(result_inst_cert, "Subject"))
					self.createSign()
			else:
				out_status_sig = _('Failed to sign file "%s".\nUnknown error.') %self._ui.sign.file_name
				
				self.__outMessage(out_status_sig, self._ui.sign.icon_failed, self._ui.sign, False)
		finally:
			self._ui.sign.btn_details.setEnabled(True)

	def verifySignature(self):
		""" Verify file with the extension .sig """
		
		try:
			if self._ui.check_sign.file_edit.text():
				os.chdir(self._ui.check_sign.currently_dir)
				#If you do not change directory to the directory
				#of the current file, the certificate file is not found
				
				#check options
				if self._ui.check_sign.check_split.checkState():
					if '.sig' in self._ui.check_sign.file_name:
						new_name = self._ui.check_sign.file_name[:self._ui.check_sign.file_name.lastIndexOf(".")] #separate extention .sig
					else:
						new_name = "new_" + self._ui.check_sign.file_name
					
					subprocess.check_call('yes | %scryptcp -verify "%s" "%s" -f "%s"' %(self.dir_crypt, self._ui.check_sign.path_name, new_name, self._ui.check_sign.file_name), shell=True)
					out_status_check_sign = _('The signature file "%s" is checked.\nThe source file is separated from the signature and saved with the name "%s"') %(self._ui.check_sign.file_name, new_name)
				else:
					subprocess.check_call('yes | %scryptcp -verify "%s" -f "%s"' %(self.dir_crypt, self._ui.check_sign.path_name, self._ui.check_sign.file_name), shell=True)
					out_status_check_sign = _('The signature file "%s" is checked.') %self._ui.check_sign.file_name
				
				if self._ui.check_sign.check_install.checkState():
					status_install_cert = subprocess.call('%scertmgr -inst -file "%s" -store uMy' %(self.dir_crypt, self._ui.check_sign.file_name), shell=True)
					out_status_check_sign += _("\nThe certificate is installed.")
				
				self.infoCert("file", self._ui.check_sign)
				self.__outMessage(out_status_check_sign, self._ui.check_sign.icon_success, self._ui.check_sign, True)
			else:
				out_status_check_sign = _("Please select the file to verify sign")
				
				self.__outMessage(out_status_check_sign, self._ui.check_sign.icon_atten, self._ui.check_sign, False)
		except subprocess.CalledProcessError:
			if self._ui.check_sign.file_edit.text() != "":
				out_status_check_sign = _("This file is not a signature")
				
				self.__outMessage(out_status_check_sign, self._ui.check_sign.icon_failed, self._ui.check_sign, False)
		finally:
			self._ui.check_sign.btn_details.setEnabled(True)
	
	def encryptFile(self):
		""" Create encrypted file with the extension .msg """
		
		if self._ui.encrypt.file_edit.text():
			os.chdir(self._ui.encrypt.currently_dir)
			
			self.infoCert("store", self._ui.encrypt)
			
			new_name = rename.translit(self._ui.encrypt.file_name) + ".enc"
			
			status_encrypt_file = subprocess.check_call('yes | %scryptcp -encr -dn "%s" -der "%s" "%s"' %(self.dir_crypt, self.name_subj, self._ui.encrypt.path_name, self._ui.encrypt.currently_dir + new_name), shell=True)
			
			out_status_encrypt_file = _('The file "%s" is encrypted in file "%s"') %(self._ui.encrypt.file_name, new_name)
			
			self.__outMessage(out_status_encrypt_file , self._ui.encrypt.icon_success, self._ui.encrypt, True)
		else:
			out_status_encrypt_file = _("Please select the file to encrypted")
			
			self.__outMessage(out_status_encrypt_file , self._ui.encrypt.icon_atten, self._ui.encrypt, False)
		
		self._ui.encrypt.btn_details.setEnabled(True)
	
	def decryptFile(self):
		""" Decrypted file """
		
		try:
			if self._ui.decrypt.file_edit.text():
				os.chdir(self._ui.decrypt.currently_dir)
				self.infoCert("token", self._ui.decrypt)
				
				if '.enc' in self._ui.decrypt.file_name:
					new_name = self._ui.decrypt.file_name[:self._ui.decrypt.file_name.lastIndexOf(".")] #separate extention .msg
				else:
					new_name = "new_" + self._ui.decrypt.file_name
				
				subprocess.check_call('yes | %scryptcp -decr -dn "%s" -start "%s" "%s"' %(self.dir_crypt, self.name_subj, self._ui.decrypt.file_name, new_name), shell=True)
				
				out_status_decrypt_file = _('The file "%s" is decrypted in file "%s"') %(self._ui.decrypt.file_name, new_name)
				
				self.__outMessage(out_status_decrypt_file, self._ui.decrypt.icon_success, self._ui.decrypt, True)
				
				if self._ui.decrypt.break_recursion: self._ui.decrypt.break_recursion = False
			else:
				out_status_decrypt_file = _("Please select the file to decrypted")
				
				self.__outMessage(out_status_decrypt_file, self._ui.decrypt.icon_atten, self._ui.decrypt, False)
		except subprocess.CalledProcessError:
			if not self._ui.decrypt.break_recursion:
				self._ui.decrypt.break_recursion = True
				#Dont't change the single quotation marks in the name of the container, else slashes must be doubled
				get_result_inst_cert = subprocess.Popen("%scertmgr -inst -cont '%s' -store uMy" %(self.dir_crypt, self._ui.decrypt.list_avail_cert.currentText()), stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
				result_inst_cert, err_result_inst_cert = get_result_inst_cert.communicate()
				if err_result_inst_cert:
					out_status_sig = _('\nError: ') + err_result_inst_cert
				
					self.__outMessage(out_status_sig, self._ui.decrypt.icon_atten, self._ui.decrypt, False)
				else:
					self._ui.encrypt.list_avail_cert.insertItems(0, self.__parseConsole(result_inst_cert, "Subject"))
					self.decryptFile()
			else:
				out_status_sig = _('Failed to decryption file "%s".\nUnknown error.') %self._ui.decrypt.file_name
				
				self.__outMessage(out_status_sig, self._ui.decrypt.icon_failed, self._ui.decrypt, False)
		finally:
			self._ui.decrypt.btn_details.setEnabled(True)
	
	def installCert(self):
		""" Installing certificate from token """
		
		index_res = []
		result = map(lambda x: subprocess.call([self.dir_crypt+"certmgr", "-inst", "-cont", x, "-store", "uMy"]), self.name_cert)
		for res in result:
			if res != 0:
				index_res.append(result.index(res))
		if not index_res:
			out_status_install_cert = _("The certificate was installed successfully")
			self.informBox(out_status_install_cert, "Information")
		else:
			out_status_install_cert = _("The installation failed.\nPlease check for the token %s in the computer and try again" %', '.join(map(lambda x: self.name_cert[x], index_res)))
			self.informBox(out_status_install_cert, "Critical")
	
	def __listToken(self):
		""" Return full list uses Token in computers """
		
		list_token = []
		for device in self.context.list_devices(subsystem='usb'):
			if 'ID_MODEL' in device and 'token' in device['ID_MODEL'].lower():
				list_token.append(device['ID_SERIAL'].replace('_', ' '))
		return list_token
	
	def __detectToken(self, device):
		""" Rechecks the presence of the token
		Partially similar to the code from the function checkProgramm """
		
		#print device.action, device.items() #for debug uses token
		
		list_comp = []
		insert_token = False
		
		if 'ID_MODEL' in device and 'token' in device['ID_MODEL'].lower():
			if device.action == 'add':
				mess_bar = _("Add %s") %device['ID_SERIAL'].replace('_', ' ')
			elif device.action == 'remove':
				mess_bar = _("Remove %s") %device['ID_SERIAL'].replace('_', ' ')
			
			if self._ui.welcome_text.isVisible():
				self._ui.note_mess.setText("<FONT SIZE = '-1'><BR><BR><BR>%s</FONT>" %mess_bar)
				#if use once repaint then temporary text is not displayed
				self._ui.repaint()
				self._ui.repaint(self._ui.main_widget.rect())
			else:
				self._ui.statusBar().showMessage(mess_bar, 0)
				self._ui.repaint(self._ui.statusBar().rect()) #repaint status bar for display temporary message
			
			#print device.action, device #for debug uses token
			
			if self.__listToken():
				insert_token = True
			if not insert_token:
				map(lambda field: field.setEnabled(False), self._ui.list_off_token) #disable components linked to signature
				list_comp.append("token")
			#check running PCSCD (needed for token)
			subprocess.call("%scsptest -keyset -enum_cont -verifyc -fq" %self.dir_crypt, shell=True) #check, automated run pcscd
			run_pcscd = subprocess.call("pidof pcscd", shell=True)
			if run_pcscd != 0 and insert_token:
				list_comp.append("pcsc")
				map(lambda field: field.setEnabled(False), self._ui.list_off_token)
			else:
				if insert_token:
					info_cert = subprocess.Popen("%scsptest -keyset -enum_cont -verifyc -fq" %self.dir_crypt, stdout=subprocess.PIPE, shell=True).communicate()[0]
					self.name_cert = self.__parseConsole(info_cert, "\\\\")
					if not self.name_cert:
						map(lambda field: field.setEnabled(False), self._ui.list_off_token) #disable components linked to signature
						list_comp.append("empty_token")
					else:
						map(lambda field: field.setEnabled(True), self._ui.list_off_token)
						
						self._ui.stat_prog_icon.setPixmap(self._ui.prog_icon_green)
						self._ui.stat_prog_text.setText(_("Token(s) %s is connected") %(', '.join(self.__listToken())))
						
						self._ui.note_mess.clear()
						self._ui.sign.list_avail_cert.clear()
						self._ui.decrypt.list_avail_cert.clear()
						
						self._ui.sign.list_avail_cert.addItems(self.name_cert)
						self._ui.decrypt.list_avail_cert.addItems(self.name_cert)
			
			self.informBox(list_comp, first_mess = True)
			self._ui.statusBar().clearMessage()
	
	def __parseConsole(self, cert, field):
		""" Separate output from the console by selecting the rows
		and pulling from it the necessary information """
		
		list_key_word = []
		
		for row in re.findall(r"%s.*" %field, cert):
			if field == "Issuer" or field == "Subject":
				#This parsing basically uses to catch name Subject
				entry = row.find("CN=")+3
				end_row = row[entry:].find(",")
				if end_row >= 0:
					key_word = row[entry:(entry + end_row)]
				else:
					key_word = row[entry:]
				
			elif field == "\\\\":
				key_word = row
			else:
				key_word = row[row.find(": ")+2:]
			
			try:
				list_key_word.append(key_word.decode('utf-8'))
			except UnicodeDecodeError:
				list_key_word.append(key_word.decode('cp1251'))
		return list_key_word
	
	def __validDate(self, date, gost):
		""" Check valid cert """
		
		message = ""
		
		current_datetime = datetime.datetime.utcnow()
		if date < current_datetime:
			message += _(" - no longer valid\n")
		if '2001' in gost:
			message += _(" - using outdated signature scheme %s valid until 31 December 2018\n") %gost
		
		if message:
			message = _("This certificate:\n") + message
			self.informBox(message, "Warning")
			return _("\n\nWARNING!\n") + message
	
	def __outMessage(self, message, icon_status, mode, info_cert = False):
		""" Create output message on display """
		
		mode.icon_status_oper.setPixmap(icon_status)
		mode.old_mes_status_oper = message
		mode.mes_status_oper.setText(mode.elided_text.elidedText(message, Qt.ElideRight, mode.parent.width(), Qt.TextWordWrap))
		if info_cert:
			mode.result.setText(message + "\n\n" + self.out_status_cert + self.__validDate(self.date_valid_cert, self.gost))
		else:
			mode.result.setText(message)
	
	def informBox(self, list_comp, type_mess = "Information", first_mess = False):
		""" Create MessageBox """
		
		if isinstance(list_comp, list):
			out_mess = ""
			if list_comp:
				for comp in list_comp:
					if comp == 'crypto':
						out_mess += _("CryptoPro is not installed in the computer.<P>Please install CryptoPro to start working in the program.<P>")
						type_mess = "Critical"
					elif comp == 'token':
						out_mess += _("Token is not installed in the computer.<P>If you want to sign or decrypt file please insert token.<P>")
						type_mess = "Information"
						self._ui.stat_prog_icon.setPixmap(self._ui.prog_icon_orange)
						self._ui.stat_prog_text.setText(_("Token is not installed in the computer"))
					elif comp == 'empty_token':
						out_mess += _("Token there is no container with the certificate.<P>Signing and decrypting files is not available.<P>")
						type_mess = "Warning"
						self._ui.stat_prog_icon.setPixmap(self._ui.prog_icon_orange)
						self._ui.stat_prog_text.setText(_("Token there is no container with the certificate"))
					elif comp == 'license':
						out_mess += _("License CryptoPro is not valid<P>")
						type_mess = "Warning"
					elif comp == 'pcsc':
						out_mess += _("PC/SC Smart Card Daemon is not running.<P>If you need working with token please run utility pcscd<P>")
						type_mess = "Critical"
						self._ui.stat_prog_icon.setPixmap(self._ui.prog_icon_red)
						self._ui.stat_prog_text.setText(_("PC/SC Smart Card Daemon is not running"))
			elif not first_mess:
				out_mess = _("All the necessary components installed and working")
		else:
			out_mess = list_comp
		
		if not first_mess:
			if type_mess == "Information":
				QMessageBox.information(self._ui, _("Information"), out_mess, QMessageBox.Ok)
			elif type_mess == "Warning":
				QMessageBox.warning(self._ui, _("Warning"), out_mess, QMessageBox.Ok)
			elif type_mess == "Critical":
				QMessageBox.critical(self._ui, _("Critical"), out_mess, QMessageBox.Ok)
		elif out_mess:
			if type_mess == "Information":
				self._ui.note_mess.setText(_("<FONT SIZE = '-1'><BR><BR><BR><B>Note: </B>%s</FONT>")%out_mess)
			elif type_mess == "Warning":
				self._ui.note_mess.setText(_("<FONT SIZE = '-1'><BR><BR><BR><B>Warning: </B>%s</FONT>")%out_mess)
			elif type_mess == "Critical":
				self._ui.note_mess.setText(_("<FONT SIZE = '-1'><BR><BR><BR><B>Attention: </B>%s</FONT>")%out_mess)

if __name__ == '__main__':
	
	print "You run the part program rosa crypto tool, please launch the main file"
